function [ J, grad ] = regularized_cost_function( theta, X, y, lambda )
%REGULARIZED_COST_FUNCTION - regularized logistic regression cost function
% Inputs:
%   w       - current weight vector 
%   X       - training data (N x d). Should include a column of ones
%   y       - labels (d x 1)
%   lambda  - regularization parameter
% Outputs:
%   J - cost
%   grad - gradient

m = size(X, 1);

% Compute regularization term (don't normalize w(1))
weight_penalty = theta(2:end)'*theta(2:end);

% Compute log loss
predictions = logistic(X*theta);
log_loss = -y'*log(predictions) - (1-y)'*log(1 - predictions);

% Combine them
J = lambda/2*weight_penalty + log_loss/m;

% Compute gradient
grad = lambda*[0; theta(2:end)] + X'*(predictions - y)/m;

end

